@@ -2,18 +2,20 @@
 namespace FSpot {
 	public delegate void AreaUpdatedHandler (object sender, Gdk.Rectangle area);
 	public delegate void AreaPreparedHandler (object sender, System.EventArgs args);
-	
+
+
 	public class AsyncPixbufLoader : System.IDisposable {
 		System.IO.Stream stream;
 		Gdk.PixbufLoader loader;		
 		string path;
 		bool area_prepared = false;
 		bool done_reading = false;
-		System.Exception error;
 		Gdk.Pixbuf pixbuf;
 		PixbufOrientation orientation;
 
-
+		private Gdk.AreaUpdatedHandler au;
+		private System.EventHandler ap;
+		private System.EventHandler ev;
 
 		//byte [] buffer = new byte [8192];
 		byte [] buffer = new byte [32768];
@@ -37,6 +39,9 @@ namespace FSpot {
 		public AsyncPixbufLoader ()
 		{
 			delay = new Delay (new GLib.IdleHandler (AsyncRead));
+			ap = new System.EventHandler (HandleAreaPrepared);
+			au = new Gdk.AreaUpdatedHandler (HandleAreaUpdated);
+			ev = new System.EventHandler (HandleClosed);
 		}
 		
 		public bool Loading
@@ -52,6 +57,14 @@ namespace FSpot {
 			}
 		}
 		
+		private void FileLoad (ImageFile img)
+		{
+			pixbuf = img.Load ();
+			done_reading = true;
+			if (Done != null)
+				Done (this, System.EventArgs.Empty);
+		}
+
 		public void Load (string filename)
 		{
 			delay.Stop ();
@@ -64,25 +77,25 @@ namespace FSpot {
 			area_prepared = false;
 			damage = Gdk.Rectangle.Zero;
 
-			try {
-				orientation = PixbufUtils.GetOrientation (filename);
-			} catch (System.Exception e) {
-				System.Console.WriteLine (e.ToString ());
-				orientation = PixbufOrientation.TopLeft;
+			ImageFile img = ImageFile.Create (filename);
+			orientation = img.Orientation;
+
+			stream = img.PixbufStream ();
+			if (stream == null) {
+				FileLoad (img);
+				return;
 			}
 
-			stream = new System.IO.FileStream (filename, System.IO.FileMode.Open, System.IO.FileAccess.Read);
-			
 			loader = new Gdk.PixbufLoader ();
-			loader.AreaPrepared += HandleAreaPrepared;
-			loader.AreaUpdated += HandleAreaUpdated;
-			loader.Closed += HandleClosed;
+			loader.AreaPrepared += ap;
+			loader.AreaUpdated += au;
+			loader.Closed += ev;
 
 			ThumbnailGenerator.Default.PushBlock ();
+			//AsyncIORead (null);
 			delay.Start ();
 		}			
 
-
 		public Gdk.PixbufLoader Loader {
 			get {
 				return loader;
@@ -105,15 +118,21 @@ namespace FSpot {
 
 		private void Close () 
 		{
+			ThumbnailGenerator.Default.PopBlock ();
+				
 			try {
 				delay.Stop ();
-				if (loader != null) {
+				if (loader != null) { 
+					loader.AreaPrepared -= ap;
+					loader.AreaUpdated -= au;
 					loader.Close ();
+					loader.Closed -= ev;
 					loader.Dispose ();
 				}
 
 				loader = null;
 			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
 				if (pixbuf != null)
 					pixbuf.Dispose ();
 
@@ -136,8 +155,8 @@ namespace FSpot {
 			if (area.Width != 0 && area.Height != 0 && AreaUpdated != null)
 				AreaUpdated (this, area);
 
+			//System.Console.WriteLine ("orig {0} tform {1}", damage.ToString (), area.ToString ());
 			damage = Gdk.Rectangle.Zero;
-			//System.Console.WriteLine (area.ToString ());
 		}
 
 		private bool AsyncRead () 
@@ -148,15 +167,20 @@ namespace FSpot {
 			do {
 				span = System.DateTime.Now - start_time;
 
-				int len = stream.Read (buffer, 0, buffer.Length);
+				int len;
 				try {
-					loader.Write (buffer, (uint)len);
+					len = stream.Read (buffer, 0, buffer.Length);
+					loader.Write (buffer, (ulong)len);
 				} catch (GLib.GException e) {
+					System.Console.WriteLine (e.ToString ());
 					pixbuf = null;
 					len = -1;
 				}
 
 				if (len <= 0) {
+					if (loader.Pixbuf == null)
+						pixbuf = null;
+
 					UpdateListeners ();
 					done_reading = true;
 					Close ();
@@ -184,9 +208,6 @@ namespace FSpot {
 
 		public Gdk.Pixbuf Pixbuf {
 			get {
-				if (!area_prepared)
-					throw new System.Exception ("Load in progress but Pixbuf not ready");
-
 				return pixbuf;
 			}
 		}
@@ -217,13 +238,8 @@ namespace FSpot {
 
 		public void Dispose ()
 		{
-			if (loader != null) {
-				loader.AreaPrepared -= HandleAreaPrepared;
-				loader.AreaUpdated -= HandleAreaUpdated;
-				loader.Closed -= HandleClosed;
-			}
 			Close ();
-			
+
 			if (pixbuf != null)
 				pixbuf.Dispose ();
 		}
