@@ -67,14 +67,19 @@ public class MainWindow {
 	[Glade.Widget] MenuItem attach_tag;
 	[Glade.Widget] MenuItem remove_tag;
 	[Glade.Widget] MenuItem find_tag;
+	
+	[Glade.Widget] Scale zoom_scale;
 
 	[Glade.Widget] VPaned info_vpaned;
 
+	[Glade.Widget] Gtk.Image near_image;
+	[Glade.Widget] Gtk.Image far_image;
+
 	PhotoVersionMenu versions_submenu;
 
 	Gtk.ToggleButton browse_button;
 	Gtk.ToggleButton view_button;
-	
+
 	InfoBox info_box;
 	FSpot.InfoDisplay info_display;
 	QueryView icon_view;
@@ -125,8 +130,7 @@ public class MainWindow {
 	{
 		this.db = db;
 
-		Mono.Posix.Catalog.Init ("f-spot", FSpot.Defines.LOCALE_DIR);
-		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", "");
+		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", "f-spot");
 		gui.Autoconnect (this);
 
 		slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
@@ -151,6 +155,7 @@ public class MainWindow {
 		tag_selection_widget.SelectionChanged += OnTagSelectionChanged;
 		tag_selection_widget.DragDataGet += HandleTagSelectionDragDataGet;
 		tag_selection_widget.DragDrop += HandleTagSelectionDragDrop;
+		tag_selection_widget.DragBegin += HandleTagSelectionDragBegin;
 		Gtk.Drag.SourceSet (tag_selection_widget, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
 				    tag_target_table, DragAction.Copy | DragAction.Move);
 
@@ -183,10 +188,15 @@ public class MainWindow {
 		view_vbox.PackStart (group_selector, false, false, 0);
 		view_vbox.ReorderChild (group_selector, 0);
 
+		FSpot.QueryDisplay query_display = new FSpot.QueryDisplay (query, tag_selection_widget);
+		view_vbox.PackStart (query_display, false, false, 0);
+		view_vbox.ReorderChild (query_display, 1);
+
 		icon_view = new QueryView (query);
 		icon_view_scrolled.Add (icon_view);
-		icon_view.SelectionChanged += HandleSelectionChanged;
+		icon_view.Selection.Changed += HandleSelectionChanged;
 		icon_view.DoubleClicked += HandleDoubleClicked;
+		icon_view.Vadjustment.ValueChanged += HandleIconViewScroll;
 		icon_view.GrabFocus ();
 
 		new FSpot.PreviewPopup (icon_view);
@@ -200,6 +210,13 @@ public class MainWindow {
 		TagMenu menu = new TagMenu (attach_tag, db.Tags);
 		menu.TagSelected += HandleAttachTagMenuSelected;
 
+		if (zoom_scale != null) {
+			zoom_scale.ValueChanged += HandleZoomScaleValueChanged;
+		}
+		
+		near_image.SetFromStock ("f-spot-stock_near", IconSize.SmallToolbar);
+		far_image.SetFromStock ("f-spot-stock_far", IconSize.SmallToolbar);
+
 		menu = new TagMenu (find_tag, db.Tags);
 		menu.TagSelected += HandleFindTagMenuSelected;
 
@@ -237,13 +254,12 @@ public class MainWindow {
 		main_window.ShowAll ();
 		main_window.Destroyed += HandleCloseCommand;
 
+		query_display.HandleChanged (query);
+
 		if (Toplevel == null)
 			Toplevel = this;
 
 		UpdateToolbar ();
-
-		if (db.Empty)
-			HandleImportCommand (null, null);
 	}
 
 	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
@@ -300,13 +316,15 @@ public class MainWindow {
 			if (view_notebook.CurrentPage != 0)
 				view_notebook.CurrentPage = 0;
 				
-			Present (photo_view.CurrentPhoto);
+			JumpTo (photo_view.Item.Index);
+			zoom_scale.Value = icon_view.ThumbnailWidth / 256.0;
 			break;
 		case ModeType.PhotoView:
 			if (view_notebook.CurrentPage != 1)
 				view_notebook.CurrentPage = 1;
 			
-			Present (icon_view.FocusCell);
+			JumpTo (icon_view.FocusCell);
+			zoom_scale.Value = photo_view.Zoom;
 			break;
 		}
 		UpdateToolbar ();
@@ -328,7 +346,6 @@ public class MainWindow {
 				view_button.Active = state;
 		}
 	}
-		
 
 	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
 	{
@@ -342,21 +359,22 @@ public class MainWindow {
 		}
 	}
 
+	private int lastTopLeftCell = -1;
 
 	public int [] SelectedIds () {
 		int [] ids = new int [0];
 
 		if (fsview != null)
-			ids = new int [] { fsview.View.CurrentPhoto };
+			ids = new int [] { fsview.View.Item.Index };
 		else {
 			switch (view_mode) {
 			case ModeType.IconView:
-				ids = icon_view.SelectedIdxs;
+				ids = icon_view.Selection.Ids;
 				break;
 			default:
 			case ModeType.PhotoView:
-				if (photo_view.View.CurrentPhotoValid ())
-					ids = new int [] { photo_view.CurrentPhoto };
+				if (photo_view.Item.IsValid)
+					ids = new int [] { photo_view.Item.Index };
 				break;
 			}
 		}
@@ -459,6 +477,37 @@ public class MainWindow {
 		}
 	}
 
+	void HandleTagSelectionDragBegin (object sender, DragBeginArgs args)
+	{
+		Tag [] tags = tag_selection_widget.TagHighlight ();
+		int len = tags.Length;
+		int size = 32;
+		int csize = size/2 + len * size / 2 + 2;
+		
+		Pixbuf container = new Pixbuf (Gdk.Colorspace.Rgb, true, 8, csize, csize);
+		container.Fill (0x00000000);
+		
+		bool use_icon = false;;
+		while (len-- > 0) {
+			Pixbuf thumbnail = tags[len].Icon;
+			
+			if (thumbnail != null) {
+				Pixbuf small = PixbufUtils.ScaleToMaxSize (thumbnail, size, size);				
+				
+				int x = len * (size/2) + (size - small.Width)/2;
+				int y = len * (size/2) + (size - small.Height)/2;
+
+				small.Composite (container, x, y, small.Width, small.Height, x, y, 1.0, 1.0, Gdk.InterpType.Nearest, 0xff);
+				small.Dispose ();
+
+				use_icon = true;
+			}
+		}
+		if (use_icon)
+			Gtk.Drag.SetIconPixbuf (args.Context, container, 0, 0);
+		container.Dispose ();
+	}
+	
 	void HandleTagSelectionDragDataGet (object sender, DragDataGetArgs args)
 	{		
 		UriList list = new UriList (SelectedPhotos ());
@@ -547,7 +596,7 @@ public class MainWindow {
 	{
 		switch (view_mode) {
 		case ModeType.PhotoView:
-			photo_view.CurrentPhoto = index;
+			photo_view.Item.Index = index;
 			break;
 		case ModeType.IconView:
 			icon_view.ScrollTo (index);
@@ -561,6 +610,37 @@ public class MainWindow {
 		JumpTo (index);
 	}
 
+	/*
+	 * Keep the glass temporal slider in sync with the user's scrolling in the icon_view
+	 */
+	private void UpdateGlass ()
+	{
+		int cell_num = icon_view.TopLeftVisibleCell();
+		
+		if (cell_num == -1 || cell_num == lastTopLeftCell)
+			return;
+
+		lastTopLeftCell = cell_num;
+		FSpot.IBrowsableItem photo = icon_view.Collection.Items [cell_num];
+#if false
+		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
+		group_selector.Adaptor.SetGlass (group_selector.Adaptor.IndexFromPhoto (photo));
+		group_selector.Adaptor.GlassSet = HandleAdaptorGlassSet;
+#else
+		/* 
+		 * FIXME this is a lame hack to get around a delagate chain.  This should 
+		 * actually operate directly on the adaptor not on the selector but I don't have 
+		 * time to fix it right now.
+		 */
+		group_selector.SetPosition (group_selector.Adaptor.IndexFromPhoto (photo));
+#endif
+	}
+	
+	void HandleIconViewScroll (object sender, EventArgs args)
+	{
+		UpdateGlass ();
+	}
+
 	//
 	// IconView Drag Handlers
 	//
@@ -651,6 +731,14 @@ public class MainWindow {
 		}
 	}
 
+	public void ImportFile (string path)
+	{
+		ImportCommand command = new ImportCommand (main_window);
+		if (command.ImportFromFile (db.Photos, path) > 0) {
+			UpdateQuery ();
+		}
+	}
+
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
@@ -666,7 +754,7 @@ public class MainWindow {
 
 			//Console.WriteLine ("Drop cell = {0} ({1},{2})", item, args.X, args.Y);
 			if (item >= 0) {
-				if (icon_view.CellIsSelected (item))
+				if (icon_view.Selection.Contains (item))
 					AttachTags (tag_selection_widget.TagHighlight (), SelectedIds());
 				else 
 					AttachTags (tag_selection_widget.TagHighlight (), new int [] {item});
@@ -692,7 +780,7 @@ public class MainWindow {
 	// IconView event handlers
 	// 
 
-	void HandleSelectionChanged (IconView view)
+	void HandleSelectionChanged (FSpot.IBrowsableCollection collection)
 	{
 		info_box.Photo = CurrentPhoto;
 		if (info_display != null)
@@ -821,6 +909,11 @@ public class MainWindow {
 	// Main menu commands
 	//
 
+	void HandleOpenCommand (object sender, EventArgs e)
+	{
+		FSpot.SingleView single = new FSpot.SingleView ();
+	}
+
 	void HandleImportCommand (object sender, EventArgs e)
 	{
 		db.Sync = false;
@@ -833,12 +926,17 @@ public class MainWindow {
 	}
 
 	void HandleImportFromCameraCommand (object sender, EventArgs e)
+	{
+		ImportCamera (null);
+	}
+
+	public void ImportCamera (string camera_device)
 	{
 		GPhotoCamera cam = new GPhotoCamera();
 
 		try {
 			int num_cameras = cam.DetectCameras();
-			int selected_cam;
+			int selected_cam = 0;
 
 			if (num_cameras < 1) {
 				HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
@@ -853,8 +951,21 @@ public class MainWindow {
 			} else if (num_cameras == 1) {
 				selected_cam = 0;
 			} else {
-				FSpot.CameraSelectionDialog camselect = new FSpot.CameraSelectionDialog (cam.CameraList);
-				selected_cam = camselect.Run ();
+				bool found = false;
+				if (camera_device != null) {
+					string port = camera_device.Remove (0, "gphoto2:".Length);
+					for (int i = 0; i < num_cameras; i++)
+						if (cam.CameraList.GetValue (i) == port) {
+							selected_cam = i;
+							found = true;
+							break;
+						}
+				}
+				
+				if (!found) {
+					FSpot.CameraSelectionDialog camselect = new FSpot.CameraSelectionDialog (cam.CameraList);
+					selected_cam = camselect.Run ();
+				}
 			}
 
 			if (selected_cam >= 0) {
@@ -921,11 +1032,6 @@ public class MainWindow {
 		new FSpot.GalleryExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 
-	void HandleExportToVfs (object sender, EventArgs args)
-	{
-		new FSpot.VfsExport (new FSpot.PhotoArray (SelectedPhotos ()));
-	}
-
 	void HandleExportToOriginal (object sender, EventArgs args)
 	{
 		new FSpot.FolderExport (new FSpot.PhotoArray (SelectedPhotos ()));
@@ -945,6 +1051,16 @@ public class MainWindow {
 		win.ShowAll ();
 	}
 
+	void HandleViewSelection (object sender, EventArgs args)
+	{
+		Gtk.Window win = new Gtk.Window ("This is a window");
+		Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
+	
+		win.Add (scroll);
+		scroll.Add (new TrayView (icon_view.Selection));
+		win.ShowAll ();
+	}
+
 	void HandleExportToFlickr (object sender, EventArgs args)
 	{
 		new FSpot.FlickrExport (new FSpot.PhotoArray (SelectedPhotos ()));
@@ -991,7 +1107,10 @@ public class MainWindow {
 			"Ewen Cheslack-Postava",
 			"Patanjali Somayaji",
 			"Matt Jones",
-			"Martin Willemoes Hansen"
+			"Martin Willemoes Hansen",
+			"Joerg Buesse",
+			"Jakub Steiner",
+			"Xavier Bouchoux"
 		};
 
                 // Translators should localize the following string
@@ -1254,8 +1373,6 @@ public class MainWindow {
 
 	private bool SlideShow ()
 	{
-		Pixbuf bg = PixbufUtils.LoadFromScreen ();
-
 		int [] ids = SelectedIds ();
 		Photo [] photos = null;
 		if (ids.Length < 2) {
@@ -1278,7 +1395,7 @@ public class MainWindow {
 			return false;
 		}
 
-		FSpot.FullSlide full = new FSpot.FullSlide (bg, photos);
+		FSpot.FullSlide full = new FSpot.FullSlide (main_window, photos);
 		full.Play ();
 		main_window.GdkWindow.Cursor = null;
 		return false;
@@ -1324,30 +1441,30 @@ public class MainWindow {
 			fsview.Destroyed += HandleFullScreenViewDestroy;
 		}
 		// FIXME this needs to be another mode like PhotoView and IconView mode.
-		fsview.View.CurrentPhoto = active;
+		fsview.View.Item.Index = active;
 
 		fsview.Show ();
 	}
-	
-	void Present (int item)
-	{
-		switch (view_mode) {
-		case ModeType.IconView:
-			icon_view.ScrollTo (item);
-			icon_view.Throb (item);
-			break;
-		case ModeType.PhotoView:
-			photo_view.CurrentPhoto = item;
-			break;
-		}
-	}
 
 	void HandleFullScreenViewDestroy (object sender, EventArgs args)
 	{
-		Present (fsview.View.CurrentPhoto);
+		JumpTo (fsview.View.Item.Index);
 		fsview = null;
 	}
 	
+	void HandleZoomScaleValueChanged (object sender, System.EventArgs args)
+	{
+		switch (view_mode) {
+		case ModeType.PhotoView:
+			if (System.Math.Abs (photo_view.Zoom - zoom_scale.Value) > System.Double.Epsilon) 
+				photo_view.Zoom = System.Math.Max (0.1, zoom_scale.Value);
+			break;
+		case ModeType.IconView:
+			icon_view.ThumbnailWidth = (int)(System.Math.Max (15, zoom_scale.Value * 256));
+			break;
+		}
+	}
+
 	void HandleZoomOut (object sender, EventArgs args)
 	{
 		switch (view_mode) {
@@ -1452,7 +1569,7 @@ public class MainWindow {
 
 	void HandleSelectNoneCommand (object sender, EventArgs args)
 	{
-		icon_view.UnselectAllCells ();
+		icon_view.Selection.Clear ();
 	}
 	
 	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
@@ -1574,8 +1691,8 @@ public class MainWindow {
 
 	void OnTagSelectionChanged (object obj)
 	{
-		SetViewMode (ModeType.IconView);
 		UpdateQuery ();
+		SetViewMode (ModeType.IconView);
 	}
 	
 	void HandleTagSelectionChanged (object obj, EventArgs args)
@@ -1585,7 +1702,7 @@ public class MainWindow {
 
 	void HandleQueryItemChanged (FSpot.IBrowsableCollection browsable, int item)
 	{
-		if (info_box.Photo == (browsable.Items[item] as Photo))
+		if (info_box.Photo == (browsable [item] as Photo))
 			info_box.Update ();
 	}
 	//
